﻿"use strict";

const sheetSizeX = 320;

const blizzardColours = [
	[ 249, 251, 255 ],
	[ 244, 246, 249 ],
	[ 237, 239, 242 ],
	[ 242, 250, 252 ],
	[ 237, 255, 255 ], // blue
	[ 252, 255, 253 ], // green
];

class BackgroundBlizzard
{
	constructor(camera, display)
	{
		this.camera = camera;
		this.display = display;

		this.sheets = [];
		for (let i = 0; i < 7; i++)
		{
			let sheet = {
				index: i,
				c: randomInt(0, blizzardColours.length),
			};
			while (i !== 0 && sheet.c === this.sheets[i - 1].c)
				sheet.c = randomInt(0, blizzardColours.length);
			this.sheets.push(sheet);
		}
		this.maxDelta = randomInt(4, 12);
		this.topOffset = alpha(50, 10, (this.maxDelta - 4) / 7);
		this.bottomOffset = alpha(150, 50, (this.maxDelta - 4) / 7);
		this.delta = randomInt(0, this.maxDelta);
		this.speedTimer = randomInt(10, 30);
	}

	render()
	{
		const context = this.display.context;
		const x = -50 - sheetSizeX + alpha(0, sheetSizeX, this.delta / this.maxDelta);
		for (let sheet of this.sheets)
		{
			const topX = x + this.topOffset + sheet.index * sheetSizeX;
			const bottomX = x - this.bottomOffset + sheet.index * sheetSizeX;
			context.fillStyle = this.display.rgba(blizzardColours[sheet.c]);
			context.beginPath();
			context.moveTo(topX, 0);
			context.lineTo(topX + sheetSizeX, 0);
			context.lineTo(bottomX + sheetSizeX, this.display.size.y);
			context.lineTo(bottomX, this.display.size.y);
			context.lineTo(topX, 0);
			context.fill();
		}
		if (++this.delta === this.maxDelta)
		{
			this.delta = 0;
			for (let i = this.sheets.length - 1; i > 0; i--)
				this.sheets[i].c = this.sheets[i - 1].c;
			this.sheets[0].c = randomInt(0, blizzardColours.length);
			while (this.sheets[0].c === this.sheets[1].c)
				this.sheets[0].c = randomInt(0, blizzardColours.length);
			if (--this.speedTimer === 0)
			{
				this.maxDelta = randomInt(4, 12);
				this.speedTimer = randomInt(10, 30);
				this.topOffset = alpha(50, 10, (this.maxDelta - 4) / 7);
				this.bottomOffset = alpha(150, 50, (this.maxDelta - 4) / 7);
			}
		}
	}
}