﻿"use strict";

const cloudRadiusX = 130;
const cloudRadiusY = 30;

const bkgSizeX = 2000;
const bkgSizeY = 1200;

class BackgroundEarth
{
	constructor(camera, display)
	{
		this.camera = camera;
		this.display = display;

		this.sunAngle = TAU * 2 / 16;

		this.clouds = [];
		const numClouds = Math.floor(Math.random() * 5) + 2;
		for (let i = 0; i < numClouds; i++)
		{
			const cloudPos = new Coord(Math.random() * bkgSizeX,
					Math.random() * (bkgSizeY - 500));
			let cloudIsGood = true;
			for (const other of this.clouds) {
				if (Math.abs(other.pos.y - cloudPos.y) < 90)
				{
					cloudIsGood = false;
					break;
				}
			}
			if (!cloudIsGood)
			{
				i--;
				continue;
			}
			let cloud = {
				pos: cloudPos,
				velocity: (Math.random() * 5 + 6) / 100,
				elements: []
			};
			this.clouds.push(cloud);
			for (let j = 0; j < 12; j++)
			{
				let x = Math.random() * cloudRadiusX * 2 - cloudRadiusX;
				let y = Math.random() * cloudRadiusY * 2 - cloudRadiusY;
				if (x ** 2 / (cloudRadiusX ** 2) + y ** 2 / (cloudRadiusY ** 2) > 1)
				{
					j--;
					continue;
				}
				let isGood = true;
				const pos = new Coord(x, y);
				for (const other of cloud.elements) {
					if (DistanceBetween(other.pos, pos) < 22)
					{
						isGood = false;
						break;
					}
				}
				if (!isGood)
				{
					j--;
					continue;
				}
				let element = {
					pos: pos,
					angle: Math.random() * TAU,
					velocity: (Math.random() * 0.6 + 0.4) * (Math.floor(Math.random() * 2) * 2 - 1) / 350,
				};
				cloud.elements.push(element);
			}
		}
	}

	render()
	{
		this.sunAngle += 0.0003;
		if (this.sunAngle >= TAU)
			this.sunAngle -= TAU;

		const plaxOff = new Coord(this.camera.pos.x * (bkgSizeX - this.display.size.x) / (50*90),
				this.camera.pos.y * (bkgSizeY - this.display.size.y) / (50*90));

		// sky
		var context = this.display.context;
		// light: 104,164,255
		// dark: 58,102,168
		let sr, sg, sb;
		let cr, cg, cb;
		let mr, mg, mb;
		if (this.sunAngle >= TAU * 2 / 14 && this.sunAngle <= TAU * 5 / 14)
		{
			// day
			sr = 104; sg = 164; sb = 255;
			cr = 247; cg = 247; cb = 247;
			mr = 156; mg = 237; mb = 168;
		}
		else if (this.sunAngle >= TAU * 8 / 14 && this.sunAngle <= TAU * 13 / 14)
		{
			// night
			sr = 58; sg = 102; sb = 168;
			cr = 175; cg = 175; cb = 175;
			mr = 126; mg = 183; mb = 135;
		}
		else
		{
			let a;
			if (this.sunAngle > TAU * 5 / 14 && this.sunAngle < TAU * 8 / 14)
				a = 1 - (this.sunAngle - TAU * 5 / 14) / (TAU * 3 / 14);
			else if (this.sunAngle < TAU * 2 / 14)
				a = this.sunAngle / (TAU * 3 / 14) + 1/3;
			else
				a = (this.sunAngle - TAU * 13 / 14) / (TAU * 3 / 14);;
			sr = alpha(58, 104, a);
			sg = alpha(102, 164, a);
			sb = alpha(168, 255, a);
			cr = alpha(175, 247, a);
			cg = alpha(175, 247, a);
			cb = alpha(175, 247, a);
			mr = alpha(126, 156, a);
			mg = alpha(183, 237, a);
			mb = alpha(135, 168, a);
		}
		context.fillStyle = `rgba(${sr}, ${sg}, ${sb}, 1)`;
		context.fillRect(0, 0, this.display.size.x, this.display.size.y);

		// sun
		let sunPos = new Coord(bkgSizeX / 2 + Math.cos(TAU - this.sunAngle) * 850 - plaxOff.x,
				bkgSizeY + Math.sin(TAU - this.sunAngle) * 900 - plaxOff.y);
		context.beginPath();
		context.fillStyle = "rgba(255, 242, 71, 1)";
		context.ellipse(sunPos.x, sunPos.y, 50, 50, 0, 0, TAU);
		context.fill();

		// clouds
		const elementColour = `rgba(${cr}, ${cg}, ${cb}, 1)`;
		for (const cloud of this.clouds) {
			cloud.pos.x += cloud.velocity;
			if (cloud.pos.x > bkgSizeX + cloudRadiusX + 50)
			{
				cloud.pos.x = -cloudRadiusX;
				cloud.pos.y = Math.random() * 250 + 40;
				cloud.velocity = (Math.random() * 5 + 6) / 140;
			}
			for (const element of cloud.elements)
			{
				element.angle += element.velocity;
				if (element.angle < 0)
					element.angle += TAU;
				else if (element.angle >= TAU)
					element.angle -= TAU;
				this._drawElement(context, cloud.pos, element, elementColour, plaxOff);
			}
		}

		// mountains
		context.beginPath();
		context.fillStyle = `rgba(${mr}, ${mg}, ${mb}, 1)`;
		let transform = context.getTransform();
		context.setTransform(1, 0, 0, 1, -plaxOff.x, -plaxOff.y);
		context.moveTo(0, bkgSizeY);
		context.lineTo(0, bkgSizeY - 130);
		context.lineTo(366, bkgSizeY - 300);
		context.lineTo(733, bkgSizeY - 150);
		context.lineTo(1100, bkgSizeY - 350);
		context.lineTo(1466, bkgSizeY - 110);
		context.lineTo(1833, bkgSizeY - 250);
		context.lineTo(2200, bkgSizeY - 90);
		context.lineTo(2200, bkgSizeY);
		context.lineTo(0, bkgSizeY)
		context.fill();
		context.setTransform(transform);
	}

	_drawElement(context, cloudPos, element, colour, plaxOff)
	{
		context.beginPath();
		context.fillStyle = colour;
		const radius = 34;
		const x = cloudPos.x + element.pos.x;
		const y = cloudPos.y + element.pos.y;
		context.moveTo(x + Math.cos(element.angle) * radius - plaxOff.x,
				y + Math.sin(element.angle) * radius - plaxOff.y);
		for (var i = 1; i <= 3; i++)
		{
			let a = TAU / 3 * i;
			context.lineTo(x + Math.cos(element.angle + a) * radius - plaxOff.x,
					y + Math.sin(element.angle + a) * radius - plaxOff.y);
		}
		context.fill();
	}
}